@@ -1,5 +1,14 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+
+## Copyright 2015-2021 PyPSA Developers
+
+## You can find the list of PyPSA Developers at
+## https://pypsa.readthedocs.io/en/latest/developers.html
+
+## PyPSA is released under the open source MIT License, see
+## https://github.com/PyPSA/PyPSA/blob/master/LICENSE.txt
+
 """
 Tools for fast Linear Problem file writing. This module contains
 
@@ -9,16 +18,23 @@ Tools for fast Linear Problem file writing. This module contains
 - solver functions which read the lp file, run the problem and return the
   solution
 
-This module supports the linear optimal power flow calculation whithout using
+This module supports the linear optimal power flow calculation without using
 pyomo (see module linopt.py)
 """
 
+__author__ = "PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html"
+__copyright__ = ("Copyright 2015-2021 PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html, "
+                 "MIT License")
+
+
 from .descriptors import Dict
 import pandas as pd
+import os
 import logging, re, io, subprocess
 import numpy as np
 from pandas import IndexSlice as idx
 from importlib.util import find_spec
+from distutils.version import LooseVersion
 
 logger = logging.getLogger(__name__)
 
@@ -26,7 +42,7 @@ logger = logging.getLogger(__name__)
 # Front end functions
 # =============================================================================
 
-def define_variables(n, lower, upper, name, attr='', axes=None, spec=''):
+def define_variables(n, lower, upper, name, attr='', axes=None, spec='', mask=None):
     """
     Defines variable(s) for pypsa-network with given lower bound(s) and upper
     bound(s). The variables are stored in the network object under n.vars with
@@ -34,8 +50,8 @@ def define_variables(n, lower, upper, name, attr='', axes=None, spec=''):
     least one of lower and upper has to be an array (including pandas) of
     shape > (1,) or axes have to define the dimensions of the variables.
 
-    Parameter
-    ---------
+    Parameters
+    ----------
     n : pypsa.Network
     lower : pd.Series/pd.DataFrame/np.array/str/float
         lower bound(s) for the variable(s)
@@ -44,8 +60,10 @@ def define_variables(n, lower, upper, name, attr='', axes=None, spec=''):
     name : str
         general name of the variable (or component which the variable is
         referring to). The variable will then be stored under:
+
             * n.vars[name].pnl if the variable is two-dimensional
             * n.vars[name].df if the variable is one-dimensional
+
         but can easily be accessed with :func:`get_var(n, name, attr)`
     attr : str default ''
         Specifying name of the variable, defines under which name the variable(s)
@@ -53,8 +71,11 @@ def define_variables(n, lower, upper, name, attr='', axes=None, spec=''):
         if one-dimensional
     axes : pd.Index or tuple of pd.Index objects, default None
         Specifies the axes and therefore the shape of the variables if bounds
-        are single strings or floats. This is helpful when mutliple variables
+        are single strings or floats. This is helpful when multiple variables
         have the same upper and lower bound.
+    mask: pd.DataFrame/np.array
+        Boolean mask with False values for variables which are skipped.
+        The shape of the mask has to match the shape the added variables.
 
 
     Example
@@ -75,32 +96,37 @@ def define_variables(n, lower, upper, name, attr='', axes=None, spec=''):
 
     Note that this is usefull for the `extra_functionality` argument.
     """
-    var = write_bound(n, lower, upper, axes)
+    var = write_bound(n, lower, upper, axes, mask)
     set_varref(n, var, name, attr, spec=spec)
     return var
 
 
-def define_binaries(n, axes, name, attr='',  spec=''):
+def define_binaries(n, axes, name, attr='',  spec='', mask=None):
     """
     Defines binary-variable(s) for pypsa-network. The variables are stored
     in the network object under n.vars with key of the variable name.
     For each entry for the pd.Series of pd.DataFrame spanned by the axes
     argument the function defines a binary.
 
-    Parameter
-    ---------
+    Parameters
+    ----------
     n : pypsa.Network
     axes : pd.Index or tuple of pd.Index objects
         Specifies the axes and therefore the shape of the variables.
     name : str
         general name of the variable (or component which the variable is
         referring to). The variable will then be stored under:
+
             * n.vars[name].pnl if the variable is two-dimensional
             * n.vars[name].df if the variable is one-dimensional
+
     attr : str default ''
         Specifying name of the variable, defines under which name the variable(s)
         are stored in n.vars[name].pnl if two-dimensional or in n.vars[name].df
         if one-dimensional
+    mask: pd.DataFrame/np.array
+        Boolean mask with False values for variables which are skipped.
+        The shape of the mask has to match the shape given by axes.
 
     See also
     ---------
@@ -112,13 +138,14 @@ def define_binaries(n, axes, name, attr='',  spec=''):
     return var
 
 
-def define_constraints(n, lhs, sense, rhs, name, attr='', axes=None, spec=''):
+def define_constraints(n, lhs, sense, rhs, name, attr='', axes=None, spec='',
+                       mask=None):
     """
     Defines constraint(s) for pypsa-network with given left hand side (lhs),
     sense and right hand side (rhs). The constraints are stored in the network
     object under n.cons with key of the constraint name. If multiple constraints
     are defined at ones, only using np.arrays, then the axes argument can be used
-    for defining the axes for the constraints (this is espececially recommended
+    for defining the axes for the constraints (this is especially recommended
     for time-dependent constraints). If one of lhs, sense and rhs is a
     pd.Series/pd.DataFrame the axes argument is not necessary.
 
@@ -146,6 +173,10 @@ def define_constraints(n, lhs, sense, rhs, name, attr='', axes=None, spec=''):
     axes: pd.Index or tuple of pd.Index objects, default None
         Specifies the axes if all of lhs, sense and rhs are np.arrays or single
         strings or floats.
+    mask: pd.DataFrame/np.array
+        Boolean mask with False values for constraints which are skipped.
+        The shape of the mask has to match the shape of the array that come out
+        when combining lhs, sense and rhs.
 
 
     Example
@@ -155,7 +186,7 @@ def define_constraints(n, lhs, sense, rhs, name, attr='', axes=None, spec=''):
     during the first 10 snapshots. We then firstly get all operational variables
     for this subset and constraint there sum to less equal 100.
 
-    >>> from pypsa.linopt import get_var, linexpr, defin_constraints
+    >>> from pypsa.linopt import get_var, linexpr, define_constraints
     >>> gas_i = n.generators.query('carrier == "Natural Gas"').index
     >>> gas_vars = get_var(n, 'Generator', 'p').loc[n.snapshots[:10], gas_i]
     >>> lhs = linexpr((1, gas_vars)).sum().sum()
@@ -166,14 +197,14 @@ def define_constraints(n, lhs, sense, rhs, name, attr='', axes=None, spec=''):
 
     >>> cons = get_var(n, 'Generator', 'gas_power_limit')
 
-    Under the hook they are stored in n.cons.Generator.pnl.gas_power_limit.
+    Under the hood they are stored in n.cons.Generator.pnl.gas_power_limit.
     For retrieving their shadow prices add the general name of the constraint
     to the keep_shadowprices argument.
 
-    Note that this is usefull for the `extra_functionality` argument.
+    Note that this is useful for the `extra_functionality` argument.
 
     """
-    con = write_constraint(n, lhs, sense, rhs, axes)
+    con = write_constraint(n, lhs, sense, rhs, axes, mask)
     set_conref(n, con, name, attr, spec=spec)
     return con
 
@@ -191,25 +222,28 @@ def _get_handlers(axes, *maybearrays):
     return axes, shape, size
 
 
-def write_bound(n, lower, upper, axes=None):
+def write_bound(n, lower, upper, axes=None, mask=None):
     """
-    Writer function for writing out mutliple variables at a time. If lower and
+    Writer function for writing out multiple variables at a time. If lower and
     upper are floats it demands to give pass axes, a tuple of (index, columns)
     or (index), for creating the variable of same upper and lower bounds.
     Return a series or frame with variable references.
     """
     axes, shape, size = _get_handlers(axes, lower, upper)
-    if not size: return pd.Series()
+    if not size: return pd.Series(dtype=float)
     n._xCounter += size
     variables = np.arange(n._xCounter - size, n._xCounter).reshape(shape)
     lower, upper = _str_array(lower), _str_array(upper)
-    n.bounds_f.write(join_exprs(lower + ' <= x' + _str_array(variables, True)
-                                + ' <= '+ upper + '\n'))
+    exprs = lower + ' <= x' + _str_array(variables, True) + ' <= '+ upper + '\n'
+    if mask is not None:
+        exprs = np.where(mask, exprs, '')
+        variables = np.where(mask, variables, -1)
+    n.bounds_f.write(join_exprs(exprs))
     return to_pandas(variables, *axes)
 
-def write_constraint(n, lhs, sense, rhs, axes=None):
+def write_constraint(n, lhs, sense, rhs, axes=None, mask=None):
     """
-    Writer function for writing out mutliple constraints to the corresponding
+    Writer function for writing out multiple constraints to the corresponding
     constraints file. If lower and upper are numpy.ndarrays it axes must not be
     None but a tuple of (index, columns) or (index).
     Return a series or frame with constraint references.
@@ -221,13 +255,16 @@ def write_constraint(n, lhs, sense, rhs, axes=None):
     if isinstance(sense, str):
         sense = '=' if sense == '==' else sense
     lhs, sense, rhs = _str_array(lhs), _str_array(sense), _str_array(rhs)
-    n.constraints_f.write(join_exprs('c' + _str_array(cons, True) + ':\n' +
-                                     lhs + sense + ' ' + rhs + '\n\n'))
+    exprs = 'c' + _str_array(cons, True) + ':\n' + lhs + sense + ' ' + rhs + '\n\n'
+    if mask is not None:
+        exprs = np.where(mask, exprs, '')
+        cons = np.where(mask, cons, -1)
+    n.constraints_f.write(join_exprs(exprs))
     return to_pandas(cons, *axes)
 
-def write_binary(n, axes):
+def write_binary(n, axes, mask=None):
     """
-    Writer function for writing out mutliple binary-variables at a time.
+    Writer function for writing out multiple binary-variables at a time.
     According to the axes it writes out binaries for each entry the pd.Series
     or pd.DataFrame spanned by axes. Returns a series or frame with variable
     references.
@@ -235,7 +272,11 @@ def write_binary(n, axes):
     axes, shape, size = _get_handlers(axes)
     n._xCounter += size
     variables = np.arange(n._xCounter - size, n._xCounter).reshape(shape)
-    n.binaries_f.write(join_exprs('x' + _str_array(variables, True) + '\n'))
+    exprs = 'x' + _str_array(variables, True) + '\n'
+    if mask is not None:
+        exprs = np.where(mask, exprs, '')
+        variables = np.where(mask, variables, -1)
+    n.binaries_f.write(join_exprs(exprs))
     return to_pandas(variables, *axes)
 
 
@@ -290,7 +331,7 @@ def align_with_static_component(n, c, attr):
     """
     Alignment of time-dependent variables with static components. If c is a
     pypsa.component name, it will sort the columns of the variable according
-    to the statid component.
+    to the static component.
     """
     if c in n.all_components and (c, attr) in n.variables.index:
         if not n.variables.pnl[c, attr]: return
@@ -350,6 +391,9 @@ def linexpr(*tuples, as_pandas=True, return_axes=False):
     if np.prod(shape):
         for coeff, var in tuples:
             expr = expr + _str_array(coeff) + ' x' + _str_array(var, True) + '\n'
+            if isinstance(expr, np.ndarray):
+                isna = np.isnan(coeff) | np.isnan(var) | (var == -1)
+                expr = np.where(isna, '', expr)
     if return_axes:
         return (expr, *axes)
     if as_pandas:
@@ -376,10 +420,13 @@ def _str_array(array, integer_string=False):
             return _to_int_str(array)
         return _to_float_str(array)
     array = np.asarray(array)
+    if array.dtype.type == np.str_:
+        array = np.asarray(array, dtype=object)
     if array.dtype < str and array.size:
         if integer_string:
-            return _v_to_int_str(np.asarray(array))
-        return _v_to_float_str(np.asarray(array))
+            array = np.nan_to_num(array, False, -1)
+            return _v_to_int_str(array)
+        return _v_to_float_str(array)
     else:
         return array
 
@@ -401,17 +448,18 @@ def _add_reference(ref_dict, df, attr, pnl=True):
         else:
             ref_dict.pnl[attr] = df
     else:
-        if ref_dict.df.empty:
-            ref_dict.df[attr] = df
-        else:
+        if attr in ref_dict.df:
             ref_dict.df = pd.concat([ref_dict.df, df.to_frame(attr)])
+        else:
+            ref_dict.df[attr] = df
+
 
 def set_varref(n, variables, c, attr, spec=''):
     """
     Sets variable references to the network.
-    One-dimensional variable references will be collected at n.vars[c].df,
-    two-dimensional varaibles in n.vars[c].pnl
-    For example:
+    One-dimensional variable references will be collected at `n.vars[c].df`,
+    two-dimensional varaibles in `n.vars[c].pnl`. For example:
+
     * nominal capacity variables for generators are stored in
       `n.vars.Generator.df.p_nom`
     * operational variables for generators are stored in
@@ -429,10 +477,11 @@ def set_varref(n, variables, c, attr, spec=''):
 
 def set_conref(n, constraints, c, attr, spec=''):
     """
-    Sets variable references to the network.
-    One-dimensional constraint references will be collected at n.cons[c].df,
-    two-dimensional in n.cons[c].pnl
+    Sets constraint references to the network.
+    One-dimensional constraint references will be collected at `n.cons[c].df`,
+    two-dimensional in `n.cons[c].pnl`
     For example:
+
     * constraints for nominal capacity variables for generators are stored in
       `n.cons.Generator.df.mu_upper`
     * operational capacity limits for generators are stored in
@@ -449,7 +498,7 @@ def set_conref(n, constraints, c, attr, spec=''):
         _add_reference(n.cons[c], constraints, attr, pnl=pnl)
 
 def get_var(n, c, attr, pop=False):
-    '''
+    """
     Retrieves variable references for a given static or time-depending
     attribute of a given component. The function looks into n.variables to
     detect whether the variable is a time-dependent or static.
@@ -466,7 +515,7 @@ def get_var(n, c, attr, pop=False):
     -------
     >>> get_var(n, 'Generator', 'p')
 
-    '''
+    """
     vvars = n.vars[c].pnl if n.variables.pnl[c, attr] else n.vars[c].df
     return vvars.pop(attr) if pop else vvars[attr]
 
@@ -572,13 +621,16 @@ def run_and_read_cbc(n, problem_fn, solution_fn, solver_logfile,
         n.basis_fn = solution_fn.replace('.sol', '.bas')
         command += f'-basisO {n.basis_fn} '
 
-    result = subprocess.run(command.split(' '), stdout=subprocess.PIPE)
-    if solver_logfile is not None:
-        print(result.stdout.decode('utf-8'), file=open(solver_logfile, 'w'))
+    if not os.path.exists(solution_fn):
+        os.mknod(solution_fn)
+
+    log = open(solver_logfile, 'w') if solver_logfile is not None else subprocess.PIPE
+    result = subprocess.Popen(command.split(' '), stdout=log)
+    result.wait()
+
+    with open(solution_fn, "r") as f:
+        data = f.readline()
 
-    f = open(solution_fn,"r")
-    data = f.readline()
-    f.close()
 
     if data.startswith("Optimal - objective value"):
         status = "ok"
@@ -594,12 +646,15 @@ def run_and_read_cbc(n, problem_fn, solution_fn, solver_logfile,
     if termination_condition != "optimal":
         return status, termination_condition, None, None, None
 
-    sol = pd.read_csv(solution_fn, header=None, skiprows=[0],
+    f = open(solution_fn,"rb")
+    trimed_sol_fn = re.sub(rb'\*\*\s+', b'', f.read())
+    f.close()
+
+    sol = pd.read_csv(io.BytesIO(trimed_sol_fn), header=None, skiprows=[0],
                       sep=r'\s+', usecols=[1,2,3], index_col=0)
     variables_b = sol.index.str[0] == 'x'
     variables_sol = sol[variables_b][2].pipe(set_int_index)
     constraints_dual = sol[~variables_b][3].pipe(set_int_index)
-
     return (status, termination_condition, variables_sol,
             constraints_dual, objective)
 
@@ -626,7 +681,8 @@ def run_and_read_glpk(n, problem_fn, solution_fn, solver_logfile,
     if (solver_options is not None) and (solver_options != {}):
         command += solver_options
 
-    subprocess.run(command.split(' '), stdout=subprocess.PIPE)
+    result = subprocess.Popen(command.split(' '), stdout=subprocess.PIPE)
+    result.wait()
 
     f = open(solution_fn)
     def read_until_break(f):
@@ -639,7 +695,7 @@ def run_and_read_glpk(n, problem_fn, solution_fn, solver_logfile,
     info = io.StringIO(''.join(read_until_break(f))[:-2])
     info = pd.read_csv(info, sep=':',  index_col=0, header=None)[1]
     termination_condition = info.Status.lower().strip()
-    objective = float(re.sub('[^0-9\.\+\-e]+', '', info.Objective))
+    objective = float(re.sub(r'[^0-9\.\+\-e]+', '', info.Objective))
 
     if termination_condition in ["optimal","integer optimal"]:
         status = "ok"
@@ -660,7 +716,7 @@ def run_and_read_glpk(n, problem_fn, solution_fn, solver_logfile,
                               .fillna(0).pipe(set_int_index)
     else:
         logger.warning("Shadow prices of MILP couldn't be parsed")
-        constraints_dual = pd.Series(index=duals.index)
+        constraints_dual = pd.Series(index=duals.index, dtype=float)
 
     sol = io.StringIO(''.join(read_until_break(f))[:-2])
     variables_sol = (pd.read_fwf(sol)[1:].set_index('Column name')
@@ -684,8 +740,13 @@ def run_and_read_cplex(n, problem_fn, solution_fn, solver_logfile,
            "Install via 'conda install -c ibmdecisionoptimization cplex' "
            "or 'pip install cplex'")
     import cplex
+    _version = LooseVersion(cplex.__version__)
     m = cplex.Cplex()
-    out = m.set_log_stream(solver_logfile)
+    if _version >= "12.10":
+        log_file_or_path = open(solver_logfile, "w")
+    else:
+        log_file_or_path = solver_logfile
+    out = m.set_log_stream(log_file_or_path)
     if solver_options is not None:
         for key, value in solver_options.items():
             param = m.parameters
@@ -697,6 +758,7 @@ def run_and_read_cplex(n, problem_fn, solution_fn, solver_logfile,
         m.start.read_basis(warmstart)
     m.solve()
     is_lp = m.problem_type[m.get_problem_type()] == 'LP'
+    if isinstance(log_file_or_path, io.IOBase): log_file_or_path.close()
 
     termination_condition = m.solution.get_status_string()
     if 'optimal' in termination_condition:
@@ -707,7 +769,11 @@ def run_and_read_cplex(n, problem_fn, solution_fn, solver_logfile,
 
     if (status == 'ok') and store_basis and is_lp:
         n.basis_fn = solution_fn.replace('.sol', '.bas')
-        m.solution.basis.write(n.basis_fn)
+        try:
+            m.solution.basis.write(n.basis_fn)
+        except cplex.exceptions.errors.CplexSolverError:
+            logger.info('No model basis stored')
+            del n.basis_fn
 
     objective = m.solution.get_objective_value()
     variables_sol = pd.Series(m.solution.get_values(), m.variables.get_names())\
@@ -734,8 +800,8 @@ def run_and_read_gurobi(n, problem_fn, solution_fn, solver_logfile,
     For more information on solver options:
     https://www.gurobi.com/documentation/{gurobi_verion}/refman/parameter_descriptions.html
     """
-    if find_spec('gurobi') is None:
-        raise ModuleNotFoundError("Optional dependency 'gurobi' not found. "
+    if find_spec('gurobipy') is None:
+        raise ModuleNotFoundError("Optional dependency 'gurobipy' not found. "
            "Install via 'conda install -c gurobi gurobi'  or follow the "
            "instructions on the documentation page "
            "https://www.gurobi.com/documentation/")
@@ -795,3 +861,74 @@ def run_and_read_gurobi(n, problem_fn, solution_fn, solver_logfile,
     del m
     return (status, termination_condition, variables_sol,
             constraints_dual, objective)
+
+
+def run_and_read_xpress(n, problem_fn, solution_fn, solver_logfile,
+                        solver_options, keep_files, warmstart=None,
+                        store_basis=True):
+    """
+    Solving function. Reads the linear problem file and passes it to
+    the Xpress solver. If the solution is successful it returns
+    variable solutions and constraint dual values. The xpress module
+    must be installed for using this function.
+
+    For more information on solver options:
+    https://www.fico.com/fico-xpress-optimization/docs/latest/solver/GUID-ACD7E60C-7852-36B7-A78A-CED0EA291CDD.html
+    """
+
+    import xpress
+
+    m = xpress.problem()
+
+    m.read(problem_fn)
+    m.setControl(solver_options)
+
+    if solver_logfile is not None:
+        m.setlogfile(solver_logfile)
+
+    if warmstart:
+        m.readbasis(warmstart)
+
+    m.solve()
+
+    if store_basis:
+        n.basis_fn = solution_fn.replace('.sol', '.bas')
+        try:
+            m.writebasis(n.basis_fn)
+        except:
+            logger.info('No model basis stored')
+            del n.basis_fn
+
+    termination_condition = m.getProbStatusString()
+
+    if termination_condition == 'mip_optimal' or \
+       termination_condition == 'lp_optimal':
+        status = 'ok'
+        termination_condition = 'optimal'
+    elif termination_condition == 'mip_unbounded' or \
+         termination_condition == 'mip_infeasible' or \
+         termination_condition == 'lp_unbounded' or \
+         termination_condition == 'lp_infeasible':
+        status = 'infeasible or unbounded'
+    else:
+        status = 'warning'
+
+    if termination_condition not in ["optimal"]:
+        return status, termination_condition, None, None, None
+
+    var = [str(v) for v in m.getVariable()]
+    variables_sol = pd.Series(m.getSolution(var), index=var).pipe(set_int_index)
+
+    try:
+        dual = [str(d) for d in m.getConstraint()]
+        constraints_dual = pd.Series(m.getDual(dual), index=dual).pipe(set_int_index)
+    except xpress.SolverError:
+        logger.warning("Shadow prices of MILP couldn't be parsed")
+        constraints_dual = pd.Series(index=dual).pipe(set_int_index)
+
+    objective = m.getObjVal()
+
+    del m
+
+    return (status, termination_condition, variables_sol,
+            constraints_dual, objective)
