@@ -1,30 +1,24 @@
-## Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS)
+## Copyright 2015-2021 PyPSA Developers
 
-## This program is free software; you can redistribute it and/or
-## modify it under the terms of the GNU General Public License as
-## published by the Free Software Foundation; either version 3 of the
-## License, or (at your option) any later version.
+## You can find the list of PyPSA Developers at
+## https://pypsa.readthedocs.io/en/latest/developers.html
 
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+## PyPSA is released under the open source MIT License, see
+## https://github.com/PyPSA/PyPSA/blob/master/LICENSE.txt
 
 """Functions for computing network clusters
 """
 
-__author__ = "Tom Brown (FIAS), Jonas Hoersch (FIAS)"
-__copyright__ = "Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS), GNU GPL 3"
+__author__ = "PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html"
+__copyright__ = ("Copyright 2015-2021 PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html, "
+                 "MIT License")
 
 import numpy as np
 import pandas as pd
 import networkx as nx
 from collections import OrderedDict, namedtuple
-from six.moves import map, range, reduce
-from six import itervalues, iteritems
+from functools import reduce
+from importlib.util import find_spec
 
 import logging
 logger = logging.getLogger(__name__)
@@ -89,7 +83,7 @@ def aggregategenerators(network, busmap, with_time=True, carriers=None, custom_s
 
     new_pnl = dict()
     if with_time:
-        for attr, df in iteritems(network.generators_t):
+        for attr, df in network.generators_t.items():
             pnl_gens_agg_b = df.columns.to_series().map(gens_agg_b)
             df_agg = df.loc[:, pnl_gens_agg_b]
             if not df_agg.empty:
@@ -125,7 +119,7 @@ def aggregateoneport(network, busmap, component, with_time=True, custom_strategi
     new_pnl = dict()
     if with_time:
         old_pnl = network.pnl(component)
-        for attr, df in iteritems(old_pnl):
+        for attr, df in old_pnl.items():
             if not df.empty:
                 pnl_df = df.groupby(grouper, axis=1).sum()
                 pnl_df.columns = _flatten_multiindex(pnl_df.columns).rename("name")
@@ -185,6 +179,7 @@ def aggregatelines(network, buses, interlines, line_length_factor=1.0):
             g=(voltage_factor * length_factor * l['g']).sum(),
             b=(voltage_factor * length_factor * l['b']).sum(),
             terrain_factor=l['terrain_factor'].mean(),
+            s_max_pu=(l['s_max_pu'] * _normed(l['s_nom'])).sum(),
             s_nom=l['s_nom'].sum(),
             s_nom_min=l['s_nom_min'].sum(),
             s_nom_max=l['s_nom_max'].sum(),
@@ -244,8 +239,8 @@ def get_clustering_from_busmap(network, busmap, with_time=True, line_length_fact
     io.import_components_from_dataframe(network_c, lines, "Line")
 
     if with_time:
-        network_c.snapshot_weightings = network.snapshot_weightings.copy()
         network_c.set_snapshots(network.snapshots)
+        network_c.snapshot_weightings = network.snapshot_weightings.copy()
 
     one_port_components = network.one_port_components.copy()
 
@@ -256,7 +251,7 @@ def get_clustering_from_busmap(network, busmap, with_time=True, line_length_fact
                                                          custom_strategies=generator_strategies)
         io.import_components_from_dataframe(network_c, generators, "Generator")
         if with_time:
-            for attr, df in iteritems(generators_pnl):
+            for attr, df in generators_pnl.items():
                 if not df.empty:
                     io.import_series_from_dataframe(network_c, df, "Generator", attr)
 
@@ -265,7 +260,7 @@ def get_clustering_from_busmap(network, busmap, with_time=True, line_length_fact
         new_df, new_pnl = aggregateoneport(network, busmap, component=one_port, with_time=with_time,
                                            custom_strategies=one_port_strategies.get(one_port, {}))
         io.import_components_from_dataframe(network_c, new_df, one_port)
-        for attr, df in iteritems(new_pnl):
+        for attr, df in new_pnl.items():
             io.import_series_from_dataframe(network_c, df, one_port, attr)
 
 
@@ -281,7 +276,7 @@ def get_clustering_from_busmap(network, busmap, with_time=True, line_length_fact
 
     if with_time:
         for c in network.iterate_components(one_port_components):
-            for attr, df in iteritems(c.pnl):
+            for attr, df in c.pnl.items():
                 if not df.empty:
                     io.import_series_from_dataframe(network_c, df, c.name, attr)
 
@@ -303,7 +298,7 @@ def get_clustering_from_busmap(network, busmap, with_time=True, line_length_fact
     io.import_components_from_dataframe(network_c, new_links, "Link")
 
     if with_time:
-        for attr, df in iteritems(network.links_t):
+        for attr, df in network.links_t.items():
             if not df.empty:
                 io.import_series_from_dataframe(network_c, df, "Link", attr)
 
@@ -313,7 +308,6 @@ def get_clustering_from_busmap(network, busmap, with_time=True, line_length_fact
 
     return Clustering(network_c, busmap, linemap, linemap_p, linemap_n)
 
-
 ################
 # Length
 
@@ -337,151 +331,146 @@ def length_clustering(network, length):
 ################
 # SpectralClustering
 
-try:
-    # available using pip as scikit-learn
+def busmap_by_spectral_clustering(network, n_clusters, **kwds):
+
+    if find_spec('sklearn') is None:
+        raise ModuleNotFoundError("Optional dependency 'sklearn' not found."
+            "Install via 'conda install -c conda-forge scikit-learn' "
+            "or 'pip install scikit-learn'")
+
     from sklearn.cluster import spectral_clustering as sk_spectral_clustering
 
-    def busmap_by_spectral_clustering(network, n_clusters, **kwds):
-        lines = network.lines.loc[:,['bus0', 'bus1']].assign(weight=network.lines.num_parallel).set_index(['bus0','bus1'])
-        lines.weight+=0.1
-        G = nx.Graph()
-        G.add_nodes_from(network.buses.index)
-        G.add_edges_from((u,v,dict(weight=w)) for (u,v),w in lines.itertuples())
-        return pd.Series(list(map(str,sk_spectral_clustering(nx.adjacency_matrix(G), n_clusters, **kwds) + 1)),
-                         index=network.buses.index)
+    lines = network.lines.loc[:,['bus0', 'bus1']].assign(weight=network.lines.num_parallel).set_index(['bus0','bus1'])
+    lines.weight+=0.1
+    G = nx.Graph()
+    G.add_nodes_from(network.buses.index)
+    G.add_edges_from((u,v,dict(weight=w)) for (u,v),w in lines.itertuples())
+    return pd.Series(list(map(str,sk_spectral_clustering(nx.adjacency_matrix(G), n_clusters, **kwds) + 1)),
+                        index=network.buses.index)
 
-    def spectral_clustering(network, n_clusters=8, **kwds):
-        busmap = busmap_by_spectral_clustering(network, n_clusters=n_clusters, **kwds)
-        return get_clustering_from_busmap(network, busmap)
-
-except ImportError:
-    pass
+def spectral_clustering(network, n_clusters=8, **kwds):
+    busmap = busmap_by_spectral_clustering(network, n_clusters=n_clusters, **kwds)
+    return get_clustering_from_busmap(network, busmap)
 
 ################
 # Louvain
 
-try:
-    # available using pip as python-louvain
+def busmap_by_louvain(network):
+
+    if find_spec('community') is None:
+        raise ModuleNotFoundError("Optional dependency 'community' not found."
+            "Install via 'conda install -c conda-forge python-louvain' "
+            "or 'pip install python-louvain'")
+
     import community
 
-    def busmap_by_louvain(network):
-        lines = network.lines.loc[:,['bus0', 'bus1']].assign(weight=network.lines.num_parallel).set_index(['bus0','bus1'])
-        lines.weight+=0.1
-        G = nx.Graph()
-        G.add_nodes_from(network.buses.index)
-        G.add_edges_from((u,v,dict(weight=w)) for (u,v),w in lines.itertuples())
-        b=community.best_partition(G)
-        list_cluster=[]
-        for i in b:
-            list_cluster.append(str(b[i]))
-        return pd.Series(list_cluster,index=network.buses.index)
-
-    def louvain_clustering(network, **kwds):
-        busmap = busmap_by_louvain(network)
-        return get_clustering_from_busmap(network, busmap)
-
-except ImportError:
-    pass
+    lines = network.lines.loc[:,['bus0', 'bus1']].assign(weight=network.lines.num_parallel).set_index(['bus0','bus1'])
+    lines.weight+=0.1
+    G = nx.Graph()
+    G.add_nodes_from(network.buses.index)
+    G.add_edges_from((u,v,dict(weight=w)) for (u,v),w in lines.itertuples())
+    b=community.best_partition(G)
+    list_cluster=[]
+    for i in b:
+        list_cluster.append(str(b[i]))
+    return pd.Series(list_cluster,index=network.buses.index)
 
+def louvain_clustering(network, **kwds):
+    busmap = busmap_by_louvain(network)
+    return get_clustering_from_busmap(network, busmap)
 
 ################
 # k-Means clustering based on bus properties
 
-try:
-    # available using pip as scikit-learn
+def busmap_by_kmeans(network, bus_weightings, n_clusters, buses_i=None, ** kwargs):
+    """
+    Create a bus map from the clustering of buses in space with a
+    weighting.
+
+    Parameters
+    ----------
+    network : pypsa.Network
+        The buses must have coordinates x,y.
+    bus_weightings : pandas.Series
+        Series of integer weights for buses, indexed by bus names.
+    n_clusters : int
+        Final number of clusters desired.
+    buses_i : None|pandas.Index
+        If not None (default), subset of buses to cluster.
+    kwargs
+        Any remaining arguments to be passed to KMeans (e.g. n_init,
+        n_jobs).
+
+    Returns
+    -------
+    busmap : pandas.Series
+        Mapping of network.buses to k-means clusters (indexed by
+        non-negative integers).
+    """
+
+    if find_spec('sklearn') is None:
+        raise ModuleNotFoundError("Optional dependency 'sklearn' not found."
+            "Install via 'conda install -c conda-forge scikit-learn' "
+            "or 'pip install scikit-learn'")
+
     from sklearn.cluster import KMeans
 
-    def busmap_by_kmeans(network, bus_weightings, n_clusters, buses_i=None, ** kwargs):
-        """
-        Create a bus map from the clustering of buses in space with a
-        weighting.
+    if buses_i is None:
+        buses_i = network.buses.index
 
-        Parameters
-        ----------
-        network : pypsa.Network
-            The buses must have coordinates x,y.
-        bus_weightings : pandas.Series
-            Series of integer weights for buses, indexed by bus names.
-        n_clusters : int
-            Final number of clusters desired.
-        buses_i : None|pandas.Index
-            If not None (default), subset of buses to cluster.
-        kwargs
-            Any remaining arguments to be passed to KMeans (e.g. n_init,
-            n_jobs).
+    # since one cannot weight points directly in the scikit-learn
+    # implementation of k-means, just add additional points at
+    # same position
+    points = (network.buses.loc[buses_i, ["x","y"]].values
+                .repeat(bus_weightings.reindex(buses_i).astype(int), axis=0))
 
-        Returns
-        -------
-        busmap : pandas.Series
-            Mapping of network.buses to k-means clusters (indexed by
-            non-negative integers).
-        """
+    kmeans = KMeans(init='k-means++', n_clusters=n_clusters, ** kwargs)
 
-        if buses_i is None:
-            buses_i = network.buses.index
+    kmeans.fit(points)
 
-        # since one cannot weight points directly in the scikit-learn
-        # implementation of k-means, just add additional points at
-        # same position
-        points = (network.buses.loc[buses_i, ["x","y"]].values
-                  .repeat(bus_weightings.reindex(buses_i).astype(int), axis=0))
+    busmap = pd.Series(data=kmeans.predict(network.buses.loc[buses_i, ["x","y"]]),
+                        index=buses_i).astype(str)
 
-        kmeans = KMeans(init='k-means++', n_clusters=n_clusters, ** kwargs)
+    return busmap
 
-        kmeans.fit(points)
+def kmeans_clustering(network, bus_weightings, n_clusters, line_length_factor=1.0, ** kwargs):
+    """
+    Cluster then network according to k-means clustering of the
+    buses.
 
-        busmap = pd.Series(data=kmeans.predict(network.buses.loc[buses_i, ["x","y"]]),
-                           index=buses_i).astype(str)
-
-        return busmap
-
-    def kmeans_clustering(network, bus_weightings, n_clusters, line_length_factor=1.0, ** kwargs):
-        """
-        Cluster then network according to k-means clustering of the
-        buses.
-
-        Buses can be weighted by an integer in the series
-        `bus_weightings`.
-
-        Note that this clustering method completely ignores the
-        branches of the network.
-
-        Parameters
-        ----------
-        network : pypsa.Network
-            The buses must have coordinates x,y.
-        bus_weightings : pandas.Series
-            Series of integer weights for buses, indexed by bus names.
-        n_clusters : int
-            Final number of clusters desired.
-        line_length_factor : float
-            Factor to multiply the crow-flies distance between new buses in order to get new
-            line lengths.
-        kwargs
-            Any remaining arguments to be passed to KMeans (e.g. n_init, n_jobs)
-
-
-        Returns
-        -------
-        Clustering : named tuple
-            A named tuple containing network, busmap and linemap
-        """
-
-        busmap = busmap_by_kmeans(network, bus_weightings, n_clusters, ** kwargs)
-        return get_clustering_from_busmap(network, busmap, line_length_factor=line_length_factor)
-
-except ImportError:
-    pass
+    Buses can be weighted by an integer in the series
+    `bus_weightings`.
 
+    Note that this clustering method completely ignores the
+    branches of the network.
 
+    Parameters
+    ----------
+    network : pypsa.Network
+        The buses must have coordinates x,y.
+    bus_weightings : pandas.Series
+        Series of integer weights for buses, indexed by bus names.
+    n_clusters : int
+        Final number of clusters desired.
+    line_length_factor : float
+        Factor to multiply the crow-flies distance between new buses in order to get new
+        line lengths.
+    kwargs
+        Any remaining arguments to be passed to KMeans (e.g. n_init, n_jobs)
 
 
+    Returns
+    -------
+    Clustering : named tuple
+        A named tuple containing network, busmap and linemap
+    """
 
+    busmap = busmap_by_kmeans(network, bus_weightings, n_clusters, ** kwargs)
+    return get_clustering_from_busmap(network, busmap, line_length_factor=line_length_factor)
 
 ################
 # Rectangular grid clustering
 
-
 def busmap_by_rectangular_grid(buses, divisions=10):
     busmap = pd.Series(0, index=buses.index)
     if isinstance(divisions, tuple):
@@ -489,7 +478,7 @@ def busmap_by_rectangular_grid(buses, divisions=10):
     else:
         divisions_x = divisions_y = divisions
     gb = buses.groupby([pd.cut(buses.x, divisions_x), pd.cut(buses.y, divisions_y)])
-    for nk, oks in enumerate(itervalues(gb.groups)):
+    for nk, oks in enumerate(gb.groups.values()):
         busmap.loc[oks] = nk
     return busmap
 
@@ -497,10 +486,6 @@ def rectangular_grid_clustering(network, divisions):
     busmap = busmap_by_rectangular_grid(network.buses, divisions)
     return get_clustering_from_busmap(network, busmap)
 
-
-
-
-
 ################
 # Reduce stubs/dead-ends, i.e. nodes with valency 1, iteratively to remove tree-like structures
 
