@@ -1,30 +1,18 @@
 
+## Copyright 2015-2021 PyPSA Developers
 
-## Copyright 2015-2018 Tom Brown (FIAS), Jonas Hoersch (FIAS), David
-## Schlachtberger (FIAS)
+## You can find the list of PyPSA Developers at
+## https://pypsa.readthedocs.io/en/latest/developers.html
 
-## This program is free software; you can redistribute it and/or
-## modify it under the terms of the GNU General Public License as
-## published by the Free Software Foundation; either version 3 of the
-## License, or (at your option) any later version.
-
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+## PyPSA is released under the open source MIT License, see
+## https://github.com/PyPSA/PyPSA/blob/master/LICENSE.txt
 
 """Optimal Power Flow functions.
 """
 
-
-from six import iteritems, itervalues, string_types
-
-
-__author__ = "Tom Brown (FIAS), Jonas Hoersch (FIAS), David Schlachtberger (FIAS)"
-__copyright__ = "Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS), David Schlachtberger (FIAS), GNU GPL 3"
+__author__ = "PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html"
+__copyright__ = ("Copyright 2015-2021 PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html, "
+                 "MIT License")
 
 import numpy as np
 import pandas as pd
@@ -42,12 +30,6 @@ import logging
 logger = logging.getLogger(__name__)
 
 
-from distutils.version import StrictVersion, LooseVersion
-try:
-    _pd_version = StrictVersion(pd.__version__)
-except ValueError:
-    _pd_version = LooseVersion(pd.__version__)
-
 from .pf import (calculate_dependent_values, find_slack_bus,
                  find_bus_controls, calculate_B_H, calculate_PTDF, find_tree,
                  find_cycles, _as_snapshots)
@@ -89,7 +71,7 @@ def define_generator_variables_constraints(network,snapshots):
     ## Define generator dispatch variables ##
 
     gen_p_bounds = {(gen,sn) : (None,None)
-                    for gen in extendable_gens_i | fixed_committable_gens_i
+                    for gen in extendable_gens_i.union(fixed_committable_gens_i)
                     for sn in snapshots}
 
     if len(fixed_gens_i):
@@ -549,7 +531,7 @@ def define_storage_variables_constraints(network,snapshots):
 
             soc[su,sn] =  [[],"==",0.]
 
-            elapsed_hours = network.snapshot_weightings[sn]
+            elapsed_hours = network.snapshot_weightings.stores[sn]
 
             if i == 0 and not sus.at[su,"cyclic_state_of_charge"]:
                 previous_state_of_charge = sus.at[su,"state_of_charge_initial"]
@@ -577,7 +559,7 @@ def define_storage_variables_constraints(network,snapshots):
             soc[su,sn][2] -= inflow.at[sn,su] * elapsed_hours
 
     for su,sn in spill_index:
-        elapsed_hours = network.snapshot_weightings.at[sn]
+        elapsed_hours = network.snapshot_weightings.stores[sn]
         storage_p_spill = model.storage_p_spill[su,sn]
         soc[su,sn][0].append((-1.*elapsed_hours,storage_p_spill))
 
@@ -658,7 +640,7 @@ def define_store_variables_constraints(network,snapshots):
 
             e[store,sn].lhs.variables.append((-1,model.store_e[store,sn]))
 
-            elapsed_hours = network.snapshot_weightings[sn]
+            elapsed_hours = network.snapshot_weightings.stores[sn]
 
             if i == 0 and not stores.at[store,"e_cyclic"]:
                 previous_e = stores.at[store,"e_initial"]
@@ -1071,7 +1053,7 @@ def define_nodal_balance_constraints(network,snapshots):
             network._p_balance[bus0,sn].variables.append((-1,network.model.passive_branch_p[bt,bn,sn]))
             network._p_balance[bus1,sn].variables.append((1,network.model.passive_branch_p[bt,bn,sn]))
 
-    power_balance = {k: LConstraint(v,"==",LExpression()) for k,v in iteritems(network._p_balance)}
+    power_balance = {k: LConstraint(v,"==",LExpression()) for k,v in network._p_balance.items()}
 
     l_constraint(network.model, "power_balance", power_balance,
                  list(network.buses.index), snapshots)
@@ -1114,7 +1096,7 @@ def define_global_constraints(network,snapshots):
                 gens = network.generators.index[network.generators.carrier == carrier]
                 c.lhs.variables.extend([(attribute
                                          * (1/network.generators.at[gen,"efficiency"])
-                                         * network.snapshot_weightings[sn],
+                                         * network.snapshot_weightings.generators[sn],
                                          network.model.generator_p[gen,sn])
                                         for gen in gens
                                         for sn in snapshots])
@@ -1177,7 +1159,7 @@ def define_linear_objective(network,snapshots):
     for sn, marginal_cost in zip(snapshots, marginal_cost_it):
         gen_mc, su_mc, st_mc, link_mc = marginal_cost
 
-        weight = network.snapshot_weightings[sn]
+        weight = network.snapshot_weightings.objective[sn]
         for gen in network.generators.index:
             coefficient = gen_mc.at[gen] * weight
             objective.variables.extend([(coefficient, model.generator_p[gen, sn])])
@@ -1231,10 +1213,6 @@ def define_linear_objective(network,snapshots):
 def extract_optimisation_results(network, snapshots, formulation="angles", free_pyomo=True,
                                  extra_postprocessing=None):
 
-    if isinstance(snapshots, pd.DatetimeIndex) and _pd_version < '0.18.0':
-        # Work around pandas bug #12050 (https://github.com/pydata/pandas/issues/12050)
-        snapshots = pd.Index(snapshots.values)
-
     allocate_series_dataframes(network, {'Generator': ['p'],
                                          'Load': ['p'],
                                          'StorageUnit': ['p', 'state_of_charge', 'spill'],
@@ -1250,17 +1228,18 @@ def extract_optimisation_results(network, snapshots, formulation="angles", free_
 
     model = network.model
 
-    duals = pd.Series(list(model.dual.values()), index=pd.Index(list(model.dual.keys())))
+    duals = pd.Series(list(model.dual.values()), index=pd.Index(list(model.dual.keys())),
+                      dtype=float)
 
     if free_pyomo:
         model.dual.clear()
 
     def clear_indexedvar(indexedvar):
-        for v in itervalues(indexedvar._data):
+        for v in indexedvar._data.values():
             v.clear()
 
     def get_values(indexedvar, free=free_pyomo):
-        s = pd.Series(indexedvar.get_values())
+        s = pd.Series(indexedvar.get_values(), dtype=float)
         if free:
             clear_indexedvar(indexedvar)
         return s
@@ -1269,7 +1248,7 @@ def extract_optimisation_results(network, snapshots, formulation="angles", free_
         df.loc[snapshots] = series.unstack(0).reindex(columns=df.columns)
 
     def get_shadows(constraint, multiind=True):
-        if len(constraint) == 0: return pd.Series()
+        if len(constraint) == 0: return pd.Series(dtype=float)
 
         index = list(constraint.keys())
         if multiind:
@@ -1362,7 +1341,9 @@ def extract_optimisation_results(network, snapshots, formulation="angles", free_
                             .map(duals))
 
             #correct for snapshot weightings
-            network.buses_t.marginal_price.loc[snapshots] = network.buses_t.marginal_price.loc[snapshots].divide(network.snapshot_weightings.loc[snapshots],axis=0)
+            network.buses_t.marginal_price.loc[snapshots] = (
+                network.buses_t.marginal_price.loc[snapshots].divide(
+                    network.snapshot_weightings.objective.loc[snapshots],axis=0))
 
         if formulation == "angles":
             set_from_series(network.buses_t.v_ang,
@@ -1449,6 +1430,9 @@ def network_lopf_build_model(network, snapshots=None, skip_pre=False,
     -------
     network.model
     """
+    if isinstance(network.snapshots, pd.MultiIndex):
+        raise NotImplementedError("Optimization with multiindexed snapshots "
+                                  "using pyomo is not supported.")
 
     if not skip_pre:
         network.determine_network_topology()
@@ -1571,7 +1555,7 @@ def network_lopf_solve(network, snapshots=None, formulation="angles", solver_opt
     else:
         args = [network.model]
 
-    if isinstance(free_memory, string_types):
+    if isinstance(free_memory, str):
         free_memory = {free_memory}
 
     if 'pypsa' in free_memory:
@@ -1583,8 +1567,8 @@ def network_lopf_solve(network, snapshots=None, formulation="angles", solver_opt
     if logger.isEnabledFor(logging.INFO):
         network.results.write()
 
-    status = network.results["Solver"][0]["Status"].key
-    termination_condition = network.results["Solver"][0]["Termination condition"].key
+    status = network.results["Solver"][0]["Status"]
+    termination_condition = network.results["Solver"][0]["Termination condition"]
 
     if status == "ok" and termination_condition == "optimal":
         logger.info("Optimization successful")
@@ -1603,9 +1587,11 @@ def network_lopf_solve(network, snapshots=None, formulation="angles", solver_opt
     return status, termination_condition
 
 def network_lopf(network, snapshots=None, solver_name="glpk", solver_io=None,
-                 skip_pre=False, extra_functionality=None, solver_logfile=None, solver_options={},
-                 keep_files=False, formulation="angles", ptdf_tolerance=0.,
-                 free_memory={},extra_postprocessing=None):
+                 skip_pre=False, extra_functionality=None,
+                 multi_investment_periods=False, solver_logfile=None,
+                 solver_options={}, keep_files=False, formulation="angles",
+                 ptdf_tolerance=0., free_memory={},
+                 extra_postprocessing=None):
     """
     Linear optimal power flow for a group of snapshots.
 
@@ -1656,6 +1642,11 @@ def network_lopf(network, snapshots=None, solver_name="glpk", solver_io=None,
     -------
     None
     """
+    if multi_investment_periods:
+        raise NotImplementedError("Multi period invesmtent is only supported for pyomo=False")
+    if (type(network.snapshots)==pd.MultiIndex):
+        raise NotImplementedError("Multi indexed snapshots is only supported for pyomo=False")
+
 
     snapshots = _as_snapshots(network, snapshots)
 
