@@ -1,51 +1,35 @@
 
+## Copyright 2015-2021 PyPSA Developers
 
-## Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS)
+## You can find the list of PyPSA Developers at
+## https://pypsa.readthedocs.io/en/latest/developers.html
 
-## This program is free software; you can redistribute it and/or
-## modify it under the terms of the GNU General Public License as
-## published by the Free Software Foundation; either version 3 of the
-## License, or (at your option) any later version.
-
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+## PyPSA is released under the open source MIT License, see
+## https://github.com/PyPSA/PyPSA/blob/master/LICENSE.txt
 
 """
 Tools for fast Pyomo linear problem building.
 
 Essentially this library replaces Pyomo expressions with more strict
 objects with a pre-defined affine structure.
-
-This code is also available as a gist
-
-https://gist.github.com/nworbmot/db3d446fa3b5c388519390e46fd5d8c3
-
-under a more permissive Apache 2.0 licence to allow sharing with other
-projects.
-
 """
 
+__author__ = "PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html"
+__copyright__ = ("Copyright 2015-2021 PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html, "
+                 "MIT License")
 
 import logging
 logger = logging.getLogger(__name__)
 
 
 from pyomo.environ import Constraint, Objective, Var, ComponentUID, minimize
+from pyomo.core.expr.numeric_expr import LinearExpression
 
 import pyomo
 from contextlib import contextmanager
-from six import iteritems
 from six.moves import cPickle as pickle
 import gc, os, tempfile
 
-__author__ = "Tom Brown (FIAS), Jonas Hoersch (FIAS)"
-__copyright__ = "Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS), GNU GPL 3"
-
 
 # =============================================================================
 # Tools for solving with pyomo
@@ -144,33 +128,12 @@ class LConstraint(object):
     def __repr__(self):
         return "{} {} {}".format(self.lhs, self.sense, self.rhs)
 
-try:
-    try:
-        # With pyomo version 5.6.2, expr_pyomo5.py has been split into three files
-        # https://github.com/Pyomo/pyomo/pull/888
-        from pyomo.core.expr.numeric_expr import LinearExpression
-    except ImportError:
-        # [5.6, 5.6.2)
-        from pyomo.core.expr.expr_pyomo5 import LinearExpression
-
-    def _build_sum_expression(variables, constant=0.):
-        expr = LinearExpression()
-        expr.linear_vars = [item[1] for item in variables]
-        expr.linear_coefs = [item[0] for item in variables]
-        expr.constant = constant
-        return expr
-
-except ImportError:
-    # - 5.6)
-    from pyomo.core.base import expr_coopr3
-
-    def _build_sum_expression(variables, constant=0.):
-        expr = expr_coopr3._SumExpression()
-        expr._args = [item[1] for item in variables]
-        expr._coef = [item[0] for item in variables]
-        expr._const = constant
-        return expr
-
+def _build_sum_expression(variables, constant=0.):
+    expr = LinearExpression()
+    expr.linear_vars = [item[1] for item in variables]
+    expr.linear_coefs = [item[0] for item in variables]
+    expr.constant = constant
+    return expr
 
 def l_constraint(model,name,constraints,*args):
     """A replacement for pyomo's Constraint that quickly builds linear
@@ -327,10 +290,10 @@ def empty_model(model):
     os.remove(fn)
     model.__setstate__(state)
 
-    for n, rule in iteritems(rules):
+    for n, rule in rules.items():
         getattr(model, n).rule = rule
 
-    for n, bound in iteritems(bounds):
+    for n, bound in bounds.items():
         getattr(model, n)._bounds_init_rule = bound
 
     logger.debug("Reloaded pyomo model")
@@ -358,7 +321,7 @@ def empty_network(network):
     with open(fn, 'rb') as f:
         panels = pickle.load(f)
     os.remove(fn)
-    for attr, pnl in iteritems(panels):
+    for attr, pnl in panels.items():
         setattr(network, attr, pnl)
 
 def patch_optsolver_free_model_before_solving(opt, model):
@@ -386,6 +349,3 @@ def patch_optsolver_record_memusage_before_solving(opt, network):
 # =============================================================================
 # Helpers for opf_lowmemory
 # =============================================================================
-
-
-
