@@ -1,27 +1,18 @@
-## Copyright 2015-2018 Tom Brown (FIAS), Jonas Hoersch (FIAS)
 
-## This program is free software; you can redistribute it and/or
-## modify it under the terms of the GNU General Public License as
-## published by the Free Software Foundation; either version 3 of the
-## License, or (at your option) any later version.
+## Copyright 2015-2021 PyPSA Developers
 
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
+## You can find the list of PyPSA Developers at
+## https://pypsa.readthedocs.io/en/latest/developers.html
 
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+## PyPSA is released under the open source MIT License, see
+## https://github.com/PyPSA/PyPSA/blob/master/LICENSE.txt
 
 """Power flow functionality.
 """
 
-from six import iterkeys
-from six.moves.collections_abc import Sequence
-
-
-__author__ = "Tom Brown (FIAS), Jonas Hoersch (FIAS), Fabian Neumann (KIT)"
-__copyright__ = "Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS), Copyright 2019 Fabian Neumann (KIT), GNU GPL 3"
+__author__ = "PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html"
+__copyright__ = ("Copyright 2015-2021 PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html, "
+                 "MIT License")
 
 import logging
 logger = logging.getLogger(__name__)
@@ -31,12 +22,12 @@ from scipy.sparse import issparse, csr_matrix, csc_matrix, hstack as shstack, vs
 from numpy import r_, ones
 from scipy.sparse.linalg import spsolve
 from numpy.linalg import norm
+from pandas.api.types import is_list_like
 
 import numpy as np
 import pandas as pd
 import networkx as nx
 
-import six
 from operator import itemgetter
 import time
 
@@ -50,14 +41,16 @@ def real(X): return np.real(X.to_numpy())
 
 def imag(X): return np.imag(X.to_numpy())
 
+
 def _as_snapshots(network, snapshots):
     if snapshots is None:
         snapshots = network.snapshots
-    if (isinstance(snapshots, six.string_types) or
-        not isinstance(snapshots, (Sequence, pd.Index))):
-        return pd.Index([snapshots])
-    else:
-        return pd.Index(snapshots)
+    if not is_list_like(snapshots):
+        snapshots = pd.Index([snapshots])
+    if not isinstance(snapshots, pd.MultiIndex):
+        snapshots = pd.Index(snapshots)
+    assert snapshots.isin(network.snapshots).all()
+    return snapshots
 
 
 def _allocate_pf_outputs(network, linear=False):
@@ -495,10 +488,10 @@ def sub_network_pf(sub_network, snapshots=None, skip_pre=False, x_tol=1e-6, use_
             # take bus-based slack weights
             slack_weights_calc = slack_weights.reindex(buses_o).pipe(normed).fillna(0)
 
-    ss = np.empty((len(snapshots), len(buses_o)), dtype=np.complex)
+    ss = np.empty((len(snapshots), len(buses_o)), dtype=complex)
     roots = np.empty((len(snapshots), len(sub_network.pvpqs) + len(sub_network.pqs) + slack_variable_b))
     iters = pd.Series(0, index=snapshots)
-    diffs = pd.Series(index=snapshots)
+    diffs = pd.Series(index=snapshots, dtype=float)
     convs = pd.Series(False, index=snapshots)
     for i, now in enumerate(snapshots):
         p = network.buses_t.p.loc[now,buses_o]
@@ -548,8 +541,8 @@ def sub_network_pf(sub_network, snapshots=None, skip_pre=False, x_tol=1e-6, use_
     v0 = V[:,buses_indexer(branch_bus0)]
     v1 = V[:,buses_indexer(branch_bus1)]
 
-    i0 = np.empty((len(snapshots), sub_network.Y0.shape[0]), dtype=np.complex)
-    i1 = np.empty((len(snapshots), sub_network.Y1.shape[0]), dtype=np.complex)
+    i0 = np.empty((len(snapshots), sub_network.Y0.shape[0]), dtype=complex)
+    i1 = np.empty((len(snapshots), sub_network.Y1.shape[0]), dtype=complex)
     for i, now in enumerate(snapshots):
         i0[i] = sub_network.Y0*V[i]
         i1[i] = sub_network.Y1*V[i]
@@ -564,7 +557,7 @@ def sub_network_pf(sub_network, snapshots=None, skip_pre=False, x_tol=1e-6, use_
         c.pnl.p1.loc[snapshots,s1t.columns] = s1t.values.real
         c.pnl.q1.loc[snapshots,s1t.columns] = s1t.values.imag
 
-    s_calc = np.empty((len(snapshots), len(buses_o)), dtype=np.complex)
+    s_calc = np.empty((len(snapshots), len(buses_o)), dtype=complex)
     for i in np.arange(len(snapshots)):
         s_calc[i] = V[i]*np.conj(sub_network.Y*V[i])
     slack_index = buses_o.get_loc(sub_network.slack_bus)
@@ -739,6 +732,8 @@ def calculate_dependent_values(network):
     apply_transformer_types(network)
 
     network.lines["v_nom"] = network.lines.bus0.map(network.buses.v_nom)
+    network.lines.loc[network.lines.carrier == "", "carrier"] = (
+        network.lines.bus0.map(network.buses.carrier))
 
     network.lines["x_pu"] = network.lines.x/(network.lines.v_nom**2)
     network.lines["r_pu"] = network.lines.r/(network.lines.v_nom**2)
@@ -762,6 +757,13 @@ def calculate_dependent_values(network):
     network.shunt_impedances["b_pu"] = network.shunt_impedances.b*network.shunt_impedances.v_nom**2
     network.shunt_impedances["g_pu"] = network.shunt_impedances.g*network.shunt_impedances.v_nom**2
 
+    network.links.loc[network.links.carrier == "", "carrier"] = (
+        network.links.bus0.map(network.buses.carrier))
+
+    network.stores.loc[network.stores.carrier == "", "carrier"] = (
+        network.stores.bus.map(network.buses.carrier))
+
+
 
 def find_slack_bus(sub_network):
     """Find the slack bus in a connected sub-network."""
@@ -1063,7 +1065,7 @@ def find_tree(sub_network, weight='x_pu'):
     for j,bus in enumerate(buses_i):
         path = nx.shortest_path(sub_network.tree,bus,tree_slack_bus)
         for i in range(len(path)-1):
-            branch = next(iterkeys(graph[path[i]][path[i+1]]))
+            branch = next(iter(graph[path[i]][path[i+1]].keys()))
             branch_i = branches_i.get_loc(branch)
             sign = +1 if branches_bus0.iat[branch_i] == path[i] else -1
             sub_network.T[branch_i,j] = sign
@@ -1096,7 +1098,7 @@ def find_cycles(sub_network, weight='x_pu'):
     for j,cycle in enumerate(cycles):
 
         for i in range(len(cycle)):
-            branch = next(iterkeys(mgraph[cycle[i]][cycle[(i+1)%len(cycle)]]))
+            branch = next(iter(mgraph[cycle[i]][cycle[(i+1)%len(cycle)]].keys()))
             branch_i = branches_i.get_loc(branch)
             sign = +1 if branches_bus0.iat[branch_i] == cycle[i] else -1
             sub_network.C[branch_i,j] += sign
