@@ -1,25 +1,18 @@
 
+## Copyright 2015-2021 PyPSA Developers
 
-## Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS)
+## You can find the list of PyPSA Developers at
+## https://pypsa.readthedocs.io/en/latest/developers.html
 
-## This program is free software; you can redistribute it and/or
-## modify it under the terms of the GNU General Public License as
-## published by the Free Software Foundation; either version 3 of the
-## License, or (at your option) any later version.
-
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+## PyPSA is released under the open source MIT License, see
+## https://github.com/PyPSA/PyPSA/blob/master/LICENSE.txt
 
 """Functions for plotting networks.
 """
 
-
-from six import string_types
+__author__ = "PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html"
+__copyright__ = ("Copyright 2015-2021 PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html, "
+                 "MIT License")
 
 import pandas as pd
 import numpy as np
@@ -29,11 +22,6 @@ import warnings
 import logging
 logger = logging.getLogger(__name__)
 
-
-__author__ = "Tom Brown (FIAS), Jonas Hoersch (FIAS), Fabian Hofmann (FIAS), Fabian Neumann (KIT)"
-__copyright__ = "Copyright 2015-2020 Tom Brown (FIAS), Jonas Hoersch (FIAS); Copyright 2019-2020 Fabian Hofmann (FIAS), Fabian Neumann (KIT), GNU GPL 3"
-
-
 import matplotlib.pyplot as plt
 from matplotlib.patches import Wedge, Circle
 from matplotlib.collections import LineCollection, PatchCollection
@@ -56,7 +44,7 @@ except ImportError:
     pltly_present = False
 
 
-def plot(n, margin=0.05, ax=None, geomap=True, projection=None,
+def plot(n, margin=None, ax=None, geomap=True, projection=None,
          bus_colors='cadetblue', bus_alpha=1, bus_sizes=2e-2, bus_cmap=None,
          line_colors='rosybrown', link_colors='darkseagreen',
          transformer_colors='orange',
@@ -146,6 +134,8 @@ def plot(n, margin=0.05, ax=None, geomap=True, projection=None,
         Collections for buses and branches.
     """
     x, y = _get_coordinates(n, layouter=layouter)
+    if boundaries is None and margin:
+        boundaries = sum(zip(*compute_bbox_with_margins(margin, x, y)), ())
 
     if geomap:
         if not cartopy_present:
@@ -163,12 +153,19 @@ def plot(n, margin=0.05, ax=None, geomap=True, projection=None,
                     'create one with: \nimport cartopy.crs as ccrs \n'
                     'fig, ax = plt.subplots('
                     'subplot_kw={"projection":ccrs.PlateCarree()})')
-        transform = draw_map_cartopy(n, x, y, ax, boundaries, margin, geomap,
-                                     color_geomap)
+        transform = draw_map_cartopy(n, x, y, ax, geomap, color_geomap)
         x, y, z = ax.projection.transform_points(transform, x.values, y.values).T
         x, y = pd.Series(x, n.buses.index), pd.Series(y, n.buses.index)
+        if boundaries is not None:
+            ax.set_extent(boundaries, crs=transform)
     elif ax is None:
         ax = plt.gca()
+    if not geomap and boundaries:
+        ax.axis(boundaries)
+
+    ax.set_aspect('equal')
+    ax.axis('off')
+    ax.set_title(title)
 
     # Plot buses:
 
@@ -192,7 +189,7 @@ def plot(n, margin=0.05, ax=None, geomap=True, projection=None,
 
         bus_sizes = bus_sizes.sort_index(level=0, sort_remaining=False)
         if geomap:
-            bus_sizes *= projected_area_factor(ax, n.srid)**2
+            bus_sizes = bus_sizes * projected_area_factor(ax, n.srid)**2
 
         patches = []
         for b_i in bus_sizes.index.levels[0]:
@@ -209,13 +206,13 @@ def plot(n, margin=0.05, ax=None, geomap=True, projection=None,
                                      360*start, 360*(start+ratio),
                                      facecolor=bus_colors[i], alpha=bus_alpha))
                 start += ratio
-        bus_collection = PatchCollection(patches, match_original=True)
+        bus_collection = PatchCollection(patches, match_original=True, zorder=5)
         ax.add_collection(bus_collection)
     else:
         c = pd.Series(bus_colors, index=n.buses.index)
         s = pd.Series(bus_sizes, index=n.buses.index, dtype="float")
         if geomap:
-            s *= projected_area_factor(ax, n.srid)**2
+            s = s * projected_area_factor(ax, n.srid)**2
 
         if bus_cmap is not None and c.dtype is np.dtype('float'):
             if isinstance(bus_cmap, str):
@@ -228,7 +225,7 @@ def plot(n, margin=0.05, ax=None, geomap=True, projection=None,
             radius = s.at[b_i]**0.5
             patches.append(Circle((x.at[b_i], y.at[b_i]), radius,
                                    facecolor=c.at[b_i], alpha=bus_alpha))
-        bus_collection = PatchCollection(patches, match_original=True)
+        bus_collection = PatchCollection(patches, match_original=True, zorder=5)
         ax.add_collection(bus_collection)
 
     # Plot branches:
@@ -316,18 +313,8 @@ def plot(n, margin=0.05, ax=None, geomap=True, projection=None,
         b_collection.set_zorder(3)
         branch_collections.append(b_collection)
 
-    bus_collection.set_zorder(5)
-
-    ax.update_datalim(compute_bbox_with_margins(margin, x, y))
-    ax.autoscale_view()
-
-    if geomap:
-        ax.outline_patch.set_visible(False)
-    else:
-        ax.set_aspect('equal')
-    ax.axis('off')
-
-    ax.set_title(title)
+    if boundaries is None:
+        ax.autoscale()
 
     return (bus_collection,) + tuple(branch_collections) + tuple(arrow_collections)
 
@@ -384,19 +371,12 @@ def projected_area_factor(ax, original_crs=4326):
                    /abs((pbounds[0] - pbounds[1])[:2].prod()))
 
 
-def draw_map_cartopy(n, x, y, ax, boundaries=None, margin=0.05,
-                     geomap=True, color_geomap=None):
-
-    if boundaries is None:
-        (x1, y1), (x2, y2) = compute_bbox_with_margins(margin, x, y)
-    else:
-        x1, x2, y1, y2 = boundaries
+def draw_map_cartopy(n, x, y, ax, geomap=True, color_geomap=None):
 
     resolution = '50m' if isinstance(geomap, bool) else geomap
     assert resolution in ['10m', '50m', '110m'], (
             "Resolution has to be one of '10m', '50m', '110m'")
     axis_transformation = get_projection_from_crs(n.srid)
-    ax.set_extent([x1, x2, y1, y2], crs=axis_transformation)
 
     if color_geomap is None:
         color_geomap = {'ocean': 'w', 'land': 'w'}
